@@ -6,6 +6,7 @@
     Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019
               Vladimír Vondruš <mosra@centrum.cz>
     Copyright © 2015 Jonathan Hale <squareys@googlemail.com>
+    Copyright © 2019 Guillaume Jacquemin <williamjcm@users.noreply.github.com>
 
     Permission is hereby granted, free of charge, to any person obtaining a
     copy of this software and associated documentation files (the "Software"),
@@ -42,6 +43,7 @@
 #include "Magnum/Magnum.h"
 #include "Magnum/Tags.h"
 #include "Magnum/Audio/visibility.h"
+#include "Magnum/Math/BoolVector.h"
 #include "MagnumExternal/OpenAL/extensions.h"
 
 namespace Magnum { namespace Audio {
@@ -57,13 +59,13 @@ Encapsulates runtime information about OpenAL extension, such as name string,
 minimal required OpenAL version and version in which the extension was adopted
 to core.
 
-See also @ref Audio::Extensions namespace, which contain compile-time information
-about OpenAL extensions.
+See also the @ref Audio::Extensions namespace, which contain compile-time
+information about OpenAL extensions.
 */
 class MAGNUM_AUDIO_EXPORT Extension {
     public:
         /** @brief All OpenAL extensions */
-        static const std::vector<Extension>& extensions();
+        static Containers::ArrayView<const Extension> extensions();
 
         /** @brief Internal unique extension index */
         constexpr std::size_t index() const { return _index; }
@@ -71,16 +73,46 @@ class MAGNUM_AUDIO_EXPORT Extension {
         /** @brief Extension string */
         constexpr const char* string() const { return _string; }
 
+        #ifndef DOXYGEN_GENERATING_OUTPUT
+        constexpr Extension(std::size_t index, const char* string): _index{index}, _string{string} {}
+        #endif
+
     private:
         /* MSVC seems to have problems with const members */
         std::size_t _index;
         const char* _string;
-
-        constexpr Extension(std::size_t index, const char* string): _index(index), _string(string) {}
 };
 
 /**
 @brief OpenAL context
+
+@section AL-Context-command-line Command-line options
+
+The context is configurable through command-line options, that can be passed
+for example from the `Platform::*Application` classes. Usage:
+
+@code{.sh}
+<application> [--magnum-help] [--magnum-disable-extensions LIST]
+              [--magnum-log default|quiet|verbose] ...
+@endcode
+
+Arguments:
+
+-   `...` --- main application arguments (see `-h` or `--help` for details)
+-   `--magnum-help` --- display this help message and exit
+-   `--magnum-disable-extensions LIST` --- API extensions to disable
+    (environment: `MAGNUM_DISABLE_EXTENSIONS`)
+-   `--magnum-log default|quiet|verbose` --- console logging
+    (environment: `MAGNUM_LOG`) (default: `default`)
+
+Note that all options are prefixed with `--magnum-` to avoid conflicts with
+options passed to the application itself. Options that don't have this prefix
+are completely ignored, see documentation of the
+@ref Utility-Arguments-delegating "Utility::Arguments" class for details.
+Moreover, `--magnum`-prefixed options unrelated to audio (such as those defined
+by @ref GL-Context-command-line "GL::Context") are ignored as well. In order to
+provide a complete help and command-line argument diagnostic, you should
+instantiate this class *after* @ref GL::Context.
  */
 class MAGNUM_AUDIO_EXPORT Context {
     public:
@@ -156,14 +188,31 @@ class MAGNUM_AUDIO_EXPORT Context {
         /**
          * @brief Constructor
          *
-         * Creates OpenAL context with given configuration.
+         * Parses command-line arguments, and creates OpenAL context with given
+         * configuration.
          */
-        #ifdef DOXYGEN_GENERATING_OUTPUT
-        explicit Context(const Configuration& configuration = Configuration());
-        #else
-        explicit Context(const Configuration& configuration);
-        explicit Context();
-        #endif
+        explicit Context(const Configuration& configuration, Int argc, const char** argv): Context(NoCreate, argc, argv) { create(configuration); }
+
+        /** @overload */
+        explicit Context(const Configuration& configuration, Int argc, char** argv): Context(configuration, argc, const_cast<const char**>(argv)) {}
+
+        /** @overload */
+        explicit Context(const Configuration& configuration, Int argc, std::nullptr_t argv): Context(configuration, argc, static_cast<const char**>(argv)) {}
+
+        /** @overload */
+        explicit Context(const Configuration& configuration): Context(configuration, 0, nullptr) {}
+
+        /** @overload */
+        explicit Context(Int argc, const char** argv);
+
+        /** @overload */
+        explicit Context(Int argc, char** argv): Context(argc, const_cast<const char**>(argv)) {}
+
+        /** @overload */
+        explicit Context(Int argc, std::nullptr_t argv): Context(argc, static_cast<const char**>(argv)) {}
+
+        /** @overload */
+        explicit Context(): Context(0, nullptr) {}
 
         /**
          * @brief Construct without creating the underlying OpenAL context
@@ -172,7 +221,16 @@ class MAGNUM_AUDIO_EXPORT Context {
          * time, for example to do a more involved configuration. Call
          * @ref create() or @ref tryCreate() to create the actual context.
          */
-        explicit Context(NoCreateT) noexcept;
+        explicit Context(NoCreateT, Int argc, const char** argv) noexcept;
+
+        /** @overload */
+        explicit Context(NoCreateT, Int argc, char** argv) noexcept: Context(NoCreate, argc, const_cast<const char**>(argv)) {}
+
+        /** @overload */
+        explicit Context(NoCreateT, Int argc, std::nullptr_t argv) noexcept: Context(NoCreate, argc, static_cast<const char**>(argv)) {}
+
+        /** @overload */
+        explicit Context(NoCreateT) noexcept: Context{NoCreate, 0, nullptr} {}
 
         /** @brief Copying is not allowed */
         Context(const Context&) = delete;
@@ -193,11 +251,6 @@ class MAGNUM_AUDIO_EXPORT Context {
         /** @brief Move assignment is not allowed */
         Context& operator=(Context&& other) = delete;
 
-        #if defined(MAGNUM_BUILD_DEPRECATED) && !defined(DOXYGEN_GENERATING_OUTPUT)
-        CORRADE_DEPRECATED("Audio::Context::current() returns reference now") Context* operator->() { return this; }
-        CORRADE_DEPRECATED("Audio::Context::current() returns reference now") operator Context*() { return this; }
-        #endif
-
         /**
          * @brief Complete the context setup and exit on failure
          *
@@ -247,13 +300,6 @@ class MAGNUM_AUDIO_EXPORT Context {
          */
         std::string hrtfSpecifierString() const;
 
-        #ifdef MAGNUM_BUILD_DEPRECATED
-        /** @brief @copybrief hrtfSpecifierString()
-         * @deprecated Use @ref hrtfSpecifierString() instead.
-         */
-        CORRADE_DEPRECATED("use hrtfSpecifierString() instead") std::string hrtfSpecifier() const { return hrtfSpecifierString(); }
-        #endif
-
         /**
          * @brief Device specifier string
          *
@@ -338,14 +384,38 @@ class MAGNUM_AUDIO_EXPORT Context {
             return _extensionStatus[extension.index()];
         }
 
+        /**
+         * @brief Whether given extension is disabled
+         *
+         * Can be used for detecting driver bug workarounds. Disabled
+         * extensions return @cpp false @ce in @ref isExtensionSupported() even
+         * if they are advertised as being supported by the driver.
+         */
+        template<class T> bool isExtensionDisabled() const {
+            return _disabledExtensions[T::Index];
+        }
+
+        /**
+         * @brief Whether given extension is disabled
+         *
+         * Can be used e.g. for listing extensions available on current
+         * hardware, but for general usage prefer @ref isExtensionDisabled() const,
+         * as it does most operations in compile time.
+         */
+        bool isExtensionDisabled(const Extension& extension) const {
+            return _disabledExtensions[extension.index()];
+        }
+
     private:
-        MAGNUM_AUDIO_LOCAL static Context* _current;
+        bool _displayInitializationLog;
 
         ALCdevice* _device;
         ALCcontext* _context;
 
-        std::bitset<Implementation::ExtensionCount> _extensionStatus;
+        Math::BoolVector<Implementation::ExtensionCount> _extensionStatus;
+        Math::BoolVector<Implementation::ExtensionCount> _disabledExtensions;
         std::vector<Extension> _supportedExtensions;
+        std::vector<std::string> _disabledExtensionStrings;
 };
 
 /**
