@@ -277,7 +277,39 @@ class MAGNUM_GL_EXPORT AbstractFramebuffer {
         AbstractFramebuffer& setViewport(const Range2Di& rectangle);
 
         /**
-         * @brief Clear specified buffers in framebuffer
+         * @brief Implementation-specific color read format
+         *
+         * The result is not cached in any way. If
+         * @gl_extension{ARB,direct_state_access} (part of OpenGL 4.5) is not
+         * available, the framebuffer is bound to some target before the
+         * operation (if not already).
+         * @see @ref implementationColorReadType(),
+         *      @fn_gl{GetNamedFramebufferParameter} with
+         *      @def_gl_keyword{IMPLEMENTATION_COLOR_READ_FORMAT},
+         *      eventually @fn_gl{BindFramebuffer} and either
+         *      @fn_gl{GetFramebufferParameter} or @fn_gl{Get} with
+         *      @def_gl{IMPLEMENTATION_COLOR_READ_FORMAT}
+         */
+        PixelFormat implementationColorReadFormat();
+
+        /**
+         * @brief Implementation-specific color read type
+         *
+         * The result is not cached in any way. If
+         * @gl_extension{ARB,direct_state_access} (part of OpenGL 4.5) is not
+         * available, the framebuffer is bound to some target before the
+         * operation (if not already).
+         * @see @ref implementationColorReadFormat(),
+         *      @fn_gl{GetNamedFramebufferParameter} with
+         *      @def_gl_keyword{IMPLEMENTATION_COLOR_READ_TYPE}, eventually
+         *      @fn_gl{BindFramebuffer} and either
+         *      @fn_gl{GetFramebufferParameter} or @fn_gl{Get} with
+         *      @def_gl{IMPLEMENTATION_COLOR_READ_TYPE}
+         */
+        PixelType implementationColorReadType();
+
+        /**
+         * @brief Clear specified buffers in the framebuffer
          * @param mask              Which buffers to clear
          * @return Reference to self (for method chaining)
          *
@@ -342,17 +374,19 @@ class MAGNUM_GL_EXPORT AbstractFramebuffer {
         #endif
 
         /**
-         * @brief Read block of pixels from framebuffer to image
+         * @brief Read block of pixels from the framebuffer to an image
          * @param rectangle         Framebuffer rectangle to read
          * @param image             Image where to put the data
          *
          * Image parameters like format and type of pixel data are taken from
          * given image. The storage is not reallocated if it is large enough to
-         * contain the new data. On OpenGL ES 2.0 and WebGL 1.0, if
-         * @ref PixelStorage::skip() is set, the functionality is emulated by
-         * adjusting the data pointer.
+         * contain the new data --- however if you want to read into existing
+         * memory or *ensure* a reallocation does not happen, use
+         * @ref read(const Range2Di&, const MutableImageView2D&) instead.
          *
-         * If @gl_extension{ARB,robustness} is available, the operation is
+         * On OpenGL ES 2.0 and WebGL 1.0, if @ref PixelStorage::skip() is set,
+         * the functionality is emulated by adjusting the data pointer. If
+         * @gl_extension{ARB,robustness} is available, the operation is
          * protected from buffer overflow.
          * @see @fn_gl{BindFramebuffer}, then @fn_gl{PixelStore} and
          *      @fn_gl_keyword{ReadPixels} or
@@ -373,6 +407,16 @@ class MAGNUM_GL_EXPORT AbstractFramebuffer {
          */
         Image2D read(const Range2Di& rectangle, Image2D&& image);
 
+        /**
+         * @brief Read block of pixels from the framebuffer to an image view
+         *
+         * Compared to @ref read(const Range2Di&, Image2D&) the function
+         * reads the pixels into the memory provided by @p image, expecting
+         * it's not @cpp nullptr @ce and its size is the same as @p rectangle
+         * size.
+         */
+        void read(const Range2Di& rectangle, const MutableImageView2D& image);
+
         #ifndef MAGNUM_TARGET_GLES2
         /**
          * @brief Read block of pixels from framebuffer to buffer image
@@ -528,13 +572,11 @@ class MAGNUM_GL_EXPORT AbstractFramebuffer {
          * @param level             Texture mip level
          * @param offset            Offset inside the texture
          *
-         * If neither @gl_extension{ARB,direct_state_access} (part of OpenGL 4.5)
-         * nor @gl_extension{EXT,direct_state_access} desktop extension is
-         * available, the texture is bound before the operation (if not
+         * If @gl_extension{ARB,direct_state_access} (part of OpenGL 4.5) is
+         * not available, the texture is bound before the operation (if not
          * already).
          * @see @ref Texture1D::setStorage(), @fn_gl{BindFramebuffer}, then
          *      @fn_gl2_keyword{CopyTextureSubImage1D,CopyTexSubImage1D},
-         *      @fn_gl_extension_keyword{CopyTextureSubImage1D,EXT,direct_state_access},
          *      eventually @fn_gl{ActiveTexture}, @fn_gl{BindTexture} and
          *      @fn_gl_keyword{CopyTexSubImage1D}
          * @requires_gl 1D textures are not available in OpenGL ES or WebGL.
@@ -549,13 +591,11 @@ class MAGNUM_GL_EXPORT AbstractFramebuffer {
          * @param level             Texture mip level
          * @param offset            Offset inside the texture
          *
-         * If neither @gl_extension{ARB,direct_state_access} (part of OpenGL 4.5)
-         * nor @gl_extension{EXT,direct_state_access} desktop extension is
-         * available, the texture is bound before the operation (if not
+         * If @gl_extension{ARB,direct_state_access} (part of OpenGL 4.5) is
+         * not available, the texture is bound before the operation (if not
          * already).
          * @see @ref Texture2D::setStorage(), @fn_gl{BindFramebuffer}, then
          *      @fn_gl2_keyword{CopyTextureSubImage2D,CopyTexSubImage2D},
-         *      @fn_gl_extension_keyword{CopyTextureSubImage2D,EXT,direct_state_access},
          *      eventually @fn_gl{ActiveTexture}, @fn_gl{BindTexture} and
          *      @fn_gl{CopyTexSubImage2D}
          */
@@ -568,13 +608,11 @@ class MAGNUM_GL_EXPORT AbstractFramebuffer {
          * @param texture           Texture where to put the data
          * @param offset            Offset inside the texture
          *
-         * If neither @gl_extension{ARB,direct_state_access} (part of OpenGL 4.5)
-         * nor @gl_extension{EXT,direct_state_access} desktop extension is
-         * available, the texture is bound before the operation (if not
+         * If @gl_extension{ARB,direct_state_access} (part of OpenGL 4.5) is
+         * not available, the texture is bound before the operation (if not
          * already).
          * @see @ref RectangleTexture::setStorage(), @fn_gl{BindFramebuffer},
          *      then @fn_gl2_keyword{CopyTextureSubImage2D,CopyTexSubImage2D},
-         *      @fn_gl_extension_keyword{CopyTextureSubImage2D,EXT,direct_state_access},
          *      eventually @fn_gl{ActiveTexture}, @fn_gl{BindTexture} and
          *      @fn_gl_keyword{CopyTexSubImage2D}
          * @requires_gl Rectangle textures are not available in OpenGL ES and
@@ -592,12 +630,11 @@ class MAGNUM_GL_EXPORT AbstractFramebuffer {
          *
          * Z coordinate of the offset is equivalent to number of texture face,
          * i.e. +X is `0` and so on, in order of (+X, -X, +Y, -Y, +Z, -Z). If
-         * neither @gl_extension{ARB,direct_state_access} (part of OpenGL 4.5) nor
-         * @gl_extension{EXT,direct_state_access} desktop extension is available,
-         * the texture is bound before the operation (if not already).
+         * @gl_extension{ARB,direct_state_access} (part of OpenGL 4.5) is not
+         * available, the texture is bound before the operation (if not
+         * already).
          * @see @ref CubeMapTexture::setStorage(), @fn_gl{BindFramebuffer},
          *      then @fn_gl2_keyword{CopyTextureSubImage3D,CopyTexSubImage3D},
-         *      @fn_gl_extension_keyword{CopyTextureSubImage2D,EXT,direct_state_access},
          *      eventually @fn_gl{ActiveTexture}, @fn_gl{BindTexture} and
          *      @fn_gl_keyword{CopyTexSubImage2D}
          */
@@ -611,13 +648,11 @@ class MAGNUM_GL_EXPORT AbstractFramebuffer {
          * @param level             Texture mip level
          * @param offset            Offset inside the texture
          *
-         * If neither @gl_extension{ARB,direct_state_access} (part of OpenGL 4.5)
-         * nor @gl_extension{EXT,direct_state_access} desktop extension is
-         * available, the texture is bound before the operation (if not
+         * If @gl_extension{ARB,direct_state_access} (part of OpenGL 4.5) is
+         * not available, the texture is bound before the operation (if not
          * already).
          * @see @ref Texture3D::setStorage(), @fn_gl{BindFramebuffer}, then
          *      @fn_gl2_keyword{CopyTextureSubImage3D,CopyTexSubImage3D},
-         *      @fn_gl_extension_keyword{CopyTextureSubImage3D,EXT,direct_state_access},
          *      eventually @fn_gl{ActiveTexture}, @fn_gl{BindTexture} and
          *      @fn_gl_keyword{CopyTexSubImage3D}
          * @requires_gles30 Extension @gl_extension{OES,texture_3D} in OpenGL
@@ -635,13 +670,11 @@ class MAGNUM_GL_EXPORT AbstractFramebuffer {
          * @param level             Texture mip level
          * @param offset            Offset inside the texture
          *
-         * If neither @gl_extension{ARB,direct_state_access} (part of OpenGL 4.5)
-         * nor @gl_extension{EXT,direct_state_access} desktop extension is
-         * available, the texture is bound before the operation (if not
+         * If @gl_extension{ARB,direct_state_access} (part of OpenGL 4.5) is
+         * not available, the texture is bound before the operation (if not
          * already).
          * @see @ref Texture1DArray::setStorage(), @fn_gl{BindFramebuffer},
          *      then @fn_gl2_keyword{CopyTextureSubImage2D,CopyTexSubImage2D},
-         *      @fn_gl_extension_keyword{CopyTextureSubImage2D,EXT,direct_state_access},
          *      eventually @fn_gl{ActiveTexture}, @fn_gl{BindTexture} and
          *      @fn_gl_keyword{CopyTexSubImage2D}
          * @requires_gl 1D array textures are not available in OpenGL ES or
@@ -658,13 +691,11 @@ class MAGNUM_GL_EXPORT AbstractFramebuffer {
          * @param level             Texture mip level
          * @param offset            Offset inside the texture
          *
-         * If neither @gl_extension{ARB,direct_state_access} (part of OpenGL 4.5)
-         * nor @gl_extension{EXT,direct_state_access} desktop extension is
-         * available, the texture is bound before the operation (if not
+         * If @gl_extension{ARB,direct_state_access} (part of OpenGL 4.5) is
+         * not available, the texture is bound before the operation (if not
          * already).
          * @see @ref Texture2DArray::setStorage(), @fn_gl{BindFramebuffer},
          *      then @fn_gl2_keyword{CopyTextureSubImage3D,CopyTexSubImage3D},
-         *      @fn_gl_extension_keyword{CopyTextureSubImage3D,EXT,direct_state_access},
          *      eventually @fn_gl{ActiveTexture}, @fn_gl{BindTexture} and
          *      @fn_gl_keyword{CopyTexSubImage3D}
          * @requires_gl30 Extension @gl_extension{EXT,texture_array}
@@ -684,13 +715,11 @@ class MAGNUM_GL_EXPORT AbstractFramebuffer {
          *
          * Z coordinate of the offset is equivalent to layer * 6 + number of
          * texture face, i.e. +X is `0` and so on, in order of (+X, -X, +Y, -Y,
-         * +Z, -Z). If neither @gl_extension{ARB,direct_state_access} (part of
-         * OpenGL 4.5) nor @gl_extension{EXT,direct_state_access} desktop
-         * available, the texture is bound before the operation (if not
-         * already).
+         * +Z, -Z). If @gl_extension{ARB,direct_state_access} (part of OpenGL
+         * 4.5) is not available, the texture is bound before the operation (if
+         * not already).
          * @see @ref CubeMapTextureArray::setStorage(), @fn_gl{BindFramebuffer},
          *      then @fn_gl2_keyword{CopyTextureSubImage3D,CopyTexSubImage3D},
-         *      @fn_gl_extension_keyword{CopyTextureSubImage3D,EXT,direct_state_access},
          *      eventually @fn_gl{ActiveTexture}, @fn_gl{BindTexture} and
          *      @fn_gl_keyword{CopyTexSubImage3D}
          * @requires_gl40 Extension @gl_extension{ARB,texture_cube_map_array}
@@ -755,13 +784,19 @@ class MAGNUM_GL_EXPORT AbstractFramebuffer {
         FramebufferTarget MAGNUM_GL_LOCAL bindImplementationSingle();
         #endif
 
+        GLenum MAGNUM_GL_LOCAL implementationColorReadFormatTypeImplementationGlobal(GLenum what);
+        #ifndef MAGNUM_TARGET_GLES
+        GLenum MAGNUM_GL_LOCAL implementationColorReadFormatTypeImplementationFramebuffer(GLenum what);
+        GLenum MAGNUM_GL_LOCAL implementationColorReadFormatTypeImplementationFramebufferDSA(GLenum what);
+        GLenum MAGNUM_GL_LOCAL implementationColorReadFormatTypeImplementationFramebufferDSAMesa(GLenum what);
+        #endif
+
         GLenum MAGNUM_GL_LOCAL checkStatusImplementationDefault(FramebufferTarget target);
         #ifdef MAGNUM_TARGET_GLES2
         GLenum MAGNUM_GL_LOCAL checkStatusImplementationSingle(FramebufferTarget);
         #endif
         #ifndef MAGNUM_TARGET_GLES
         GLenum MAGNUM_GL_LOCAL checkStatusImplementationDSA(FramebufferTarget target);
-        GLenum MAGNUM_GL_LOCAL checkStatusImplementationDSAEXT(FramebufferTarget target);
         #endif
 
         #ifndef MAGNUM_TARGET_GLES2
@@ -781,7 +816,6 @@ class MAGNUM_GL_EXPORT AbstractFramebuffer {
         void MAGNUM_GL_LOCAL drawBuffersImplementationDefault(GLsizei count, const GLenum* buffers);
         #ifndef MAGNUM_TARGET_GLES
         void MAGNUM_GL_LOCAL drawBuffersImplementationDSA(GLsizei count, const GLenum* buffers);
-        void MAGNUM_GL_LOCAL drawBuffersImplementationDSAEXT(GLsizei count, const GLenum* buffers);
         #endif
         #else
         void MAGNUM_GL_LOCAL drawBuffersImplementationEXT(GLsizei count, const GLenum* buffers);
@@ -793,7 +827,6 @@ class MAGNUM_GL_EXPORT AbstractFramebuffer {
         #ifndef MAGNUM_TARGET_GLES
         void MAGNUM_GL_LOCAL drawBufferImplementationDefault(GLenum buffer);
         void MAGNUM_GL_LOCAL drawBufferImplementationDSA(GLenum buffer);
-        void MAGNUM_GL_LOCAL drawBufferImplementationDSAEXT(GLenum buffer);
         #endif
 
         #if !(defined(MAGNUM_TARGET_WEBGL) && defined(MAGNUM_TARGET_GLES2))
@@ -801,7 +834,6 @@ class MAGNUM_GL_EXPORT AbstractFramebuffer {
         #endif
         #ifndef MAGNUM_TARGET_GLES
         void MAGNUM_GL_LOCAL readBufferImplementationDSA(GLenum buffer);
-        void MAGNUM_GL_LOCAL readBufferImplementationDSAEXT(GLenum buffer);
         #endif
 
         static void MAGNUM_GL_LOCAL readImplementationDefault(const Range2Di& rectangle, PixelFormat format, PixelType type, std::size_t dataSize, GLvoid* data);
@@ -812,14 +844,12 @@ class MAGNUM_GL_EXPORT AbstractFramebuffer {
         #ifndef MAGNUM_TARGET_GLES
         static void MAGNUM_GL_LOCAL copySub1DImplementationDefault(const Range2Di& rectangle, AbstractTexture& texture, Int level, Int offset);
         static void MAGNUM_GL_LOCAL copySub1DImplementationDSA(const Range2Di& rectangle, AbstractTexture& texture, Int level, Int offset);
-        static void MAGNUM_GL_LOCAL copySub1DImplementationDSAEXT(const Range2Di& rectangle, AbstractTexture& texture, Int level, Int offset);
         #endif
 
         static void MAGNUM_GL_LOCAL copySub2DImplementationDefault(const Range2Di& rectangle, AbstractTexture& texture, GLenum textureTarget, Int level, const Vector2i& offset);
         #ifndef MAGNUM_TARGET_GLES
         static void MAGNUM_GL_LOCAL copySub2DImplementationDSA(const Range2Di& rectangle, AbstractTexture& texture, GLenum textureTarget, Int level, const Vector2i& offset);
         static void MAGNUM_GL_LOCAL copySubCubeMapImplementationDSA(const Range2Di& rectangle, AbstractTexture& texture, GLenum textureTarget, Int level, const Vector2i& offset);
-        static void MAGNUM_GL_LOCAL copySub2DImplementationDSAEXT(const Range2Di& rectangle, AbstractTexture& texture, GLenum textureTarget, Int level, const Vector2i& offset);
         #endif
 
         #if !(defined(MAGNUM_TARGET_GLES2) && defined(MAGNUM_TARGET_WEBGL))
@@ -827,7 +857,6 @@ class MAGNUM_GL_EXPORT AbstractFramebuffer {
         #endif
         #ifndef MAGNUM_TARGET_GLES
         static void MAGNUM_GL_LOCAL copySub3DImplementationDSA(const Range2Di& rectangle, AbstractTexture& texture, Int level, const Vector3i& offset);
-        static void MAGNUM_GL_LOCAL copySub3DImplementationDSAEXT(const Range2Di& rectangle, AbstractTexture& texture, Int level, const Vector3i& offset);
         #endif
 
         void MAGNUM_GL_LOCAL invalidateImplementationNoOp(GLsizei, const GLenum*);
@@ -850,49 +879,6 @@ CORRADE_ENUMSET_OPERATORS(FramebufferClearMask)
 CORRADE_ENUMSET_OPERATORS(FramebufferBlitMask)
 #endif
 
-}
-
-#ifdef MAGNUM_BUILD_DEPRECATED
-/* Note: needs to be prefixed with Magnum:: otherwise Doxygen can't find it */
-
-/** @brief @copybrief GL::FramebufferClear
- * @deprecated Use @ref GL::FramebufferClear instead.
- */
-typedef CORRADE_DEPRECATED("use GL::FramebufferClear instead") Magnum::GL::FramebufferClear FramebufferClear;
-
-/** @brief @copybrief GL::FramebufferClearMask
- * @deprecated Use @ref GL::FramebufferClearMask instead.
- */
-typedef CORRADE_DEPRECATED("use GL::FramebufferClearMask instead") Magnum::GL::FramebufferClearMask FramebufferClearMask;
-
-#if !(defined(MAGNUM_TARGET_WEBGL) && defined(MAGNUM_TARGET_GLES2))
-/** @brief @copybrief GL::FramebufferBlit
- * @deprecated Use @ref GL::FramebufferBlit instead.
- */
-typedef CORRADE_DEPRECATED("use GL::FramebufferBlit instead") Magnum::GL::FramebufferBlit FramebufferBlit;
-
-/** @brief @copybrief GL::FramebufferBlitMask
- * @deprecated Use @ref GL::FramebufferBlitMask instead.
- */
-typedef CORRADE_DEPRECATED("use GL::FramebufferBlitMask instead") Magnum::GL::FramebufferBlitMask FramebufferBlitMask;
-
-/** @brief @copybrief GL::FramebufferBlitFilter
- * @deprecated Use @ref GL::FramebufferBlitFilter instead.
- */
-typedef CORRADE_DEPRECATED("use GL::FramebufferBlitFilter instead") Magnum::GL::FramebufferBlitFilter FramebufferBlitFilter;
-#endif
-
-/** @brief @copybrief GL::FramebufferTarget
- * @deprecated Use @ref GL::FramebufferTarget instead.
- */
-typedef CORRADE_DEPRECATED("use GL::FramebufferTarget instead") Magnum::GL::FramebufferTarget FramebufferTarget;
-
-/** @brief @copybrief GL::AbstractFramebuffer
- * @deprecated Use @ref GL::AbstractFramebuffer instead.
- */
-typedef CORRADE_DEPRECATED("use GL::AbstractFramebuffer instead") Magnum::GL::AbstractFramebuffer AbstractFramebuffer;
-#endif
-
-}
+}}
 
 #endif
