@@ -44,42 +44,62 @@ namespace Magnum { namespace GL {
 @brief Framebuffer
 
 Unlike @ref DefaultFramebuffer, which is used for on-screen rendering, this
-class is used for off-screen rendering, usable either in windowless
-applications, texture generation or for various post-processing effects.
+class is used for off-screen rendering, usable in windowless applications, for
+texture generation or for various post-processing effects.
 
-@section GL-Framebuffer-usage Example usage
+@section GL-Framebuffer-usage Basic usage
 
-See @ref GL-DefaultFramebuffer-usage "DefaultFramebuffer documentation" for
-introduction. Imagine you have shader with multiple outputs (e.g. for deferred
-rendering). You want to render them off-screen to textures and then use the
-textures for actual on-screen rendering. First you need to create the
-framebuffer with the same viewport as default framebuffer and attach textures
-and renderbuffers to desired outputs:
+A basic setup for rendering to a texture of a desired `size` might look like
+below. Apart from a color attachment, there's also a depth/stencil attachment
+in order to make depth test work properly. If you render 2D or don't need to
+use a depth/stencil test, you can have just a color attachment. It's possible
+to attach either a @ref Texture or a @ref Renderbuffer --- a texture allows
+you to read the rendered output later from a shader, while a renderbuffer
+allows you to only @ref read() or @ref blit() from it. In modern desktop OpenGL
+there's not much reason to use @ref Renderbuffer anymore, however in OpenGL ES
+and WebGL due to various texture format restrictions, renderbuffers are still
+the more flexible option if you don't need to use the result in a shader.
 
-@snippet MagnumGL.cpp Framebuffer-usage-attach
+@snippet MagnumGL.cpp Framebuffer-usage
 
-Then you need to map outputs of your shader to color attachments in the
-framebuffer:
+Rendering then usually consists of switching between different framebuffers
+using @ref bind() and reusing the rendered texture in subsequent draws:
 
-@snippet MagnumGL.cpp Framebuffer-usage-map
+@snippet MagnumGL.cpp Framebuffer-usage-rendering
 
-The actual @ref Platform::Sdl2Application::drawEvent() "drawEvent()" might look
-like this. First you clear all buffers you need, perform drawing to off-screen
-framebuffer, then bind the default and render the textures on screen:
+@section GL-Framebuffer-usage-multisample Multisampled rendering
 
-@snippet MagnumGL-framebuffer.cpp Framebuffer-usage-draw
+Another use case for custom framebuffers is multisampled rendering --- as
+you're not always allowed to control the MSAA setting on a default framebuffer,
+or you might want your rendered texture to be multisampled as well:
+
+@snippet MagnumGL.cpp Framebuffer-usage-multisample
+
+Here @ref Renderbuffer gets used for the color attachment as well. While it's
+possible to achieve the same with a @ref MultisampleTexture2D, support for it
+is rather sparse on OpenGL ES and completely nonexistent on WebGL or
+macOS / iOS.
+
+@section GL-Framebuffer-usage-multiple-output Multiple fragment shader outputs
+
+In a deferred rendering setup for example, a shader usually has more than one
+output. That's finally where non-zero @ref ColorAttachment and @ref mapForDraw()
+gets used. In builtin shaders this is also how the
+@ref Shaders::Flat::ColorOutput / @ref Shaders::Flat::ObjectIdOutput etc. get
+used:
+
+@snippet MagnumGL.cpp Framebuffer-usage-deferred
 
 @section GL-Framebuffer-performance-optimizations Performance optimizations
 
 See also @ref GL-AbstractFramebuffer-performance-optimization "relevant section in AbstractFramebuffer".
 
-If either @gl_extension{ARB,direct_state_access} (part of OpenGL 4.5) or
-@gl_extension{EXT,direct_state_access} desktop extension is available, functions
-@ref checkStatus(), @ref mapForDraw(), @ref mapForRead(), @ref invalidate(),
-@ref attachRenderbuffer(), @ref attachTexture(), @ref attachCubeMapTexture(),
-@ref attachTextureLayer() and @ref detach() use DSA to avoid unnecessary calls
-to @fn_gl{BindFramebuffer}. See their respective documentation for more
-information.
+If @gl_extension{ARB,direct_state_access} (part of OpenGL 4.5) is not
+available, functions @ref checkStatus(), @ref mapForDraw(), @ref mapForRead(),
+@ref invalidate(), @ref attachRenderbuffer(), @ref attachTexture(),
+@ref attachCubeMapTexture(), @ref attachTextureLayer() and @ref detach() use
+DSA to avoid unnecessary calls to @fn_gl{BindFramebuffer}. See their respective
+documentation for more information.
 
 @requires_gl30 Extension @gl_extension{ARB,framebuffer_object}
 */
@@ -359,6 +379,8 @@ class MAGNUM_GL_EXPORT Framebuffer: public AbstractFramebuffer, public AbstractO
          *
          * This function can be safely used for constructing (and later
          * destructing) objects even without any OpenGL context being active.
+         * However note that this is a low-level and a potentially dangerous
+         * API, see the documentation of @ref NoCreate for alternatives.
          * @see @ref Framebuffer(const Range2Di&), @ref wrap()
          */
         explicit Framebuffer(NoCreateT) noexcept: AbstractFramebuffer{{}, {}, {}} {}
@@ -441,9 +463,8 @@ class MAGNUM_GL_EXPORT Framebuffer: public AbstractFramebuffer, public AbstractO
          * @brief Check framebuffer status
          * @param target    Target for which check the status
          *
-         * If neither @gl_extension{ARB,direct_state_access} (part of OpenGL 4.5)
-         * nor @gl_extension{EXT,direct_state_access} desktop extension is
-         * available, the framebuffer is bound before the operation (if not
+         * If @gl_extension{ARB,direct_state_access} (part of OpenGL 4.5) is
+         * not available, the framebuffer is bound before the operation (if not
          * already).
          *
          * The @p target parameter is ignored on OpenGL ES 2.0 if none of
@@ -451,7 +472,6 @@ class MAGNUM_GL_EXPORT Framebuffer: public AbstractFramebuffer, public AbstractO
          * or @gl_extension{NV,framebuffer_blit} is available and also on WebGL
          * 1.0.
          * @see @fn_gl2_keyword{CheckNamedFramebufferStatus,CheckFramebufferStatus},
-         *      @fn_gl_extension_keyword{CheckNamedFramebufferStatus,EXT,direct_state_access},
          *      eventually @fn_gl{BindFramebuffer} and @fn_gl_keyword{CheckFramebufferStatus}
          */
         Status checkStatus(FramebufferTarget target);
@@ -490,14 +510,12 @@ class MAGNUM_GL_EXPORT Framebuffer: public AbstractFramebuffer, public AbstractO
          *
          * @snippet MagnumGL.cpp Framebuffer-mapForDraw
          *
-         * If neither @gl_extension{ARB,direct_state_access} (part of OpenGL 4.5)
-         * nor @gl_extension{EXT,direct_state_access} desktop extension is
-         * available, the framebuffer is bound before the operation (if not
+         * If @gl_extension{ARB,direct_state_access} (part of OpenGL 4.5) is
+         * not available, the framebuffer is bound before the operation (if not
          * already).
          * @see @ref maxDrawBuffers(), @ref maxDualSourceDrawBuffers(),
          *      @ref maxColorAttachments(), @ref mapForRead(),
          *      @fn_gl2_keyword{NamedFramebufferDrawBuffers,DrawBuffers},
-         *      @fn_gl_extension_keyword{FramebufferDrawBuffers,EXT,direct_state_access},
          *      eventually @fn_gl{BindFramebuffer} and @fn_gl_keyword{DrawBuffers}
          * @requires_gles30 Extension @gl_extension{EXT,draw_buffers} or
          *      @gl_extension{NV,draw_buffers} in OpenGL ES 2.0.
@@ -507,20 +525,18 @@ class MAGNUM_GL_EXPORT Framebuffer: public AbstractFramebuffer, public AbstractO
         Framebuffer& mapForDraw(std::initializer_list<std::pair<UnsignedInt, DrawAttachment>> attachments);
 
         /**
-         * @brief Map shader output to attachment
+         * @brief Map shader output to an attachment
          * @param attachment        Draw attachment
          * @return Reference to self (for method chaining)
          *
          * Similar to above function, can be used in cases when shader has
          * only one (unnamed) output.
          *
-         * If neither @gl_extension{ARB,direct_state_access} (part of OpenGL 4.5)
-         * nor @gl_extension{EXT,direct_state_access} desktop extension is
-         * available, the framebuffer is bound before the operation (if not
+         * If @gl_extension{ARB,direct_state_access} (part of OpenGL 4.5) is
+         * not available, the framebuffer is bound before the operation (if not
          * already).
          * @see @ref maxColorAttachments(), @ref mapForRead(),
          *      @fn_gl2_keyword{NamedFramebufferDrawBuffer,DrawBuffer},
-         *      @fn_gl_extension_keyword{FramebufferDrawBuffer,EXT,direct_state_access},
          *      eventually @fn_gl{BindFramebuffer} and @fn_gl_keyword{DrawBuffer}
          *      (or @fn_gl_keyword{DrawBuffers} in OpenGL ES)
          * @requires_gles30 Extension @gl_extension{EXT,draw_buffers} or
@@ -536,15 +552,13 @@ class MAGNUM_GL_EXPORT Framebuffer: public AbstractFramebuffer, public AbstractO
          * @param attachment        Color attachment
          * @return Reference to self (for method chaining)
          *
-         * If neither @gl_extension{ARB,direct_state_access} (part of OpenGL 4.5)
-         * nor @gl_extension{EXT,direct_state_access} desktop extension is
-         * available, the framebuffer is bound before the operation (if not
+         * If @gl_extension{ARB,direct_state_access} (part of OpenGL 4.5) is
+         * not available, the framebuffer is bound before the operation (if not
          * already).
          * @see @ref mapForDraw(), @fn_gl2_keyword{NamedFramebufferReadBuffer,ReadBuffer},
-         *      @fn_gl_extension_keyword{FramebufferReadBuffer,EXT,direct_state_access},
          *      eventually @fn_gl{BindFramebuffer} and @fn_gl_keyword{ReadBuffer}
-         * @requires_gles30 Extension @gl_extension2{NV,read_buffer,GL_NV_read_buffer}
-         *      in OpenGL ES 2.0.
+         * @requires_gles30 Extension @gl_extension{NV,read_buffer} in OpenGL
+         *      ES 2.0.
          * @requires_webgl20 Framebuffer read mapping is not available in WebGL
          *      1.0.
          */
@@ -598,12 +612,10 @@ class MAGNUM_GL_EXPORT Framebuffer: public AbstractFramebuffer, public AbstractO
          * @param renderbuffer      Renderbuffer
          * @return Reference to self (for method chaining)
          *
-         * If neither @gl_extension{ARB,direct_state_access} (part of OpenGL 4.5)
-         * nor @gl_extension{EXT,direct_state_access} desktop extension is
-         * available, the framebuffer is bound before the operation (if not
+         * If @gl_extension{ARB,direct_state_access} (part of OpenGL 4.5) is
+         * not available, the framebuffer is bound before the operation (if not
          * already).
          * @see @ref detach(), @fn_gl2_keyword{NamedFramebufferRenderbuffer,FramebufferRenderbuffer},
-         *      @fn_gl_extension_keyword{NamedFramebufferRenderbuffer,EXT,direct_state_access},
          *      eventually @fn_gl{BindFramebuffer} and @fn_gl_keyword{FramebufferRenderbuffer}
          */
         Framebuffer& attachRenderbuffer(BufferAttachment attachment, Renderbuffer& renderbuffer);
@@ -616,13 +628,11 @@ class MAGNUM_GL_EXPORT Framebuffer: public AbstractFramebuffer, public AbstractO
          * @param level             Mip level
          * @return Reference to self (for method chaining)
          *
-         * If neither @gl_extension{ARB,direct_state_access} (part of OpenGL 4.5)
-         * nor @gl_extension{EXT,direct_state_access} desktop extension is
-         * available, the framebuffer is bound before the operation (if not
+         * If @gl_extension{ARB,direct_state_access} (part of OpenGL 4.5) is
+         * not available, the framebuffer is bound before the operation (if not
          * already).
          * @see @ref detach(), @ref attachCubeMapTexture(),
          *      @fn_gl2_keyword{NamedFramebufferTexture,FramebufferTexture},
-         *      @fn_gl_extension_keyword{NamedFramebufferTexture1D,EXT,direct_state_access},
          *      eventually @fn_gl{BindFramebuffer} and
          *      @fn_gl2_keyword{FramebufferTexture1D,FramebufferTexture}
          * @requires_gl Only 2D and 3D textures are available in OpenGL ES and
@@ -638,13 +648,11 @@ class MAGNUM_GL_EXPORT Framebuffer: public AbstractFramebuffer, public AbstractO
          * @param level             Mip level
          * @return Reference to self (for method chaining)
          *
-         * If neither @gl_extension{ARB,direct_state_access} (part of OpenGL 4.5)
-         * nor @gl_extension{EXT,direct_state_access} desktop extension is
-         * available, the framebuffer is bound before the operation (if not
+         * If @gl_extension{ARB,direct_state_access} (part of OpenGL 4.5) is
+         * not available, the framebuffer is bound before the operation (if not
          * already).
          * @see @ref detach(), @ref attachCubeMapTexture(),
          *      @fn_gl2_keyword{NamedFramebufferTexture,FramebufferTexture},
-         *      @fn_gl_extension_keyword{NamedFramebufferTexture2D,EXT,direct_state_access},
          *      eventually @fn_gl{BindFramebuffer} and
          *      @fn_gl2_keyword{FramebufferTexture2D,FramebufferTexture}
          * @requires_gles30 Extension @gl_extension{OES,fbo_render_mipmap} to
@@ -681,13 +689,11 @@ class MAGNUM_GL_EXPORT Framebuffer: public AbstractFramebuffer, public AbstractO
          * @param level             Mip level
          * @return Reference to self (for method chaining)
          *
-         * If neither @gl_extension{ARB,direct_state_access} (part of OpenGL 4.5)
-         * nor @gl_extension{EXT,direct_state_access} desktop extension is
-         * available, the framebuffer is bound before the operation (if not
+         * If @gl_extension{ARB,direct_state_access} (part of OpenGL 4.5) is
+         * not available, the framebuffer is bound before the operation (if not
          * already).
          * @see @ref detach(), @ref attachTexture(),
          *      @fn_gl2_keyword{NamedFramebufferTextureLayer,FramebufferTextureLayer},
-         *      @fn_gl_extension_keyword{NamedFramebufferTexture2D,EXT,direct_state_access},
          *      eventually @fn_gl{BindFramebuffer} and
          *      @fn_gl2_keyword{FramebufferTexture2D,FramebufferTexture}
          * @requires_gles30 Extension @gl_extension{OES,fbo_render_mipmap} to
@@ -706,12 +712,10 @@ class MAGNUM_GL_EXPORT Framebuffer: public AbstractFramebuffer, public AbstractO
          * @param layer             Layer
          * @return Reference to self (for method chaining)
          *
-         * If neither @gl_extension{ARB,direct_state_access} (part of OpenGL 4.5)
-         * nor @gl_extension{EXT,direct_state_access} desktop extension is
-         * available, the framebuffer is bound before the operation (if not
+         * If @gl_extension{ARB,direct_state_access} (part of OpenGL 4.5) is
+         * not available, the framebuffer is bound before the operation (if not
          * already).
          * @see @ref detach(), @fn_gl2_keyword{NamedFramebufferTextureLayer,FramebufferTextureLayer},
-         *      @fn_gl_extension_keyword{NamedFramebufferTextureLayer,EXT,direct_state_access},
          *      eventually @fn_gl{BindFramebuffer} and
          *      @fn_gl2_keyword{FramebufferTextureLayer,FramebufferTexture}
          *      or @fn_gl_extension_keyword{FramebufferTexture3D,OES,texture_3D}
@@ -773,13 +777,11 @@ class MAGNUM_GL_EXPORT Framebuffer: public AbstractFramebuffer, public AbstractO
          * @return Reference to self (for method chaining)
          *
          * Attaches whole texture with all layers addressable using `gl_Layer`
-         * in geometry shader. If neither @gl_extension{ARB,direct_state_access}
-         * (part of OpenGL 4.5) nor @gl_extension{EXT,direct_state_access} desktop
-         * extension is available, the framebuffer is bound before the
+         * in geometry shader. If @gl_extension{ARB,direct_state_access} (part
+         * of OpenGL 4.5) is not available, the framebuffer is bound before the
          * operation (if not already).
          * @see @ref detach(), @ref attachTexture(),
          *      @fn_gl2_keyword{NamedFramebufferTexture,FramebufferTexture},
-         *      @fn_gl_extension_keyword{NamedFramebufferTexture,EXT,direct_state_access},
          *      eventually @fn_gl{BindFramebuffer} and
          *      @fn_gl_keyword{FramebufferTexture}
          * @requires_gl32 Extension @gl_extension{ARB,geometry_shader4}
@@ -845,14 +847,12 @@ class MAGNUM_GL_EXPORT Framebuffer: public AbstractFramebuffer, public AbstractO
          * @param attachment        Buffer attachment
          * @return Reference to self (for method chaining)
          *
-         * If neither @gl_extension{ARB,direct_state_access} (part of OpenGL 4.5)
-         * nor @gl_extension{EXT,direct_state_access} desktop extension is
-         * available, the framebuffer is bound before the operation (if not
+         * If @gl_extension{ARB,direct_state_access} (part of OpenGL 4.5) is
+         * not available, the framebuffer is bound before the operation (if not
          * already).
          * @see @ref attachRenderbuffer(), @ref attachTexture(),
          *      @ref attachCubeMapTexture(), @ref attachTextureLayer(),
          *      @fn_gl2_keyword{NamedFramebufferRenderbuffer,FramebufferRenderbuffer},
-         *      @fn_gl_extension_keyword{NamedFramebufferRenderbuffer,EXT,direct_state_access},
          *      eventually @fn_gl{BindFramebuffer} and
          *      @fn_gl_keyword{FramebufferRenderbuffer}
          */
@@ -899,18 +899,15 @@ class MAGNUM_GL_EXPORT Framebuffer: public AbstractFramebuffer, public AbstractO
         void MAGNUM_GL_LOCAL renderbufferImplementationDefault(BufferAttachment attachment, GLuint renderbufferId);
         #ifndef MAGNUM_TARGET_GLES
         void MAGNUM_GL_LOCAL renderbufferImplementationDSA(BufferAttachment attachment, GLuint renderbufferId);
-        void MAGNUM_GL_LOCAL renderbufferImplementationDSAEXT(BufferAttachment attachment, GLuint renderbufferId);
         #endif
 
         #ifndef MAGNUM_TARGET_GLES
         void MAGNUM_GL_LOCAL texture1DImplementationDefault(BufferAttachment attachment, GLuint textureId, GLint level);
-        void MAGNUM_GL_LOCAL texture1DImplementationDSAEXT(BufferAttachment attachment, GLuint textureId, GLint level);
         #endif
 
         void MAGNUM_GL_LOCAL texture2DImplementationDefault(BufferAttachment attachment, GLenum textureTarget, GLuint textureId, GLint level);
         #ifndef MAGNUM_TARGET_GLES
         void MAGNUM_GL_LOCAL texture2DImplementationDSA(BufferAttachment attachment, GLenum textureTarget, GLuint textureId, GLint level);
-        void MAGNUM_GL_LOCAL texture2DImplementationDSAEXT(BufferAttachment attachment, GLenum textureTarget, GLuint textureId, GLint level);
         void MAGNUM_GL_LOCAL textureCubeMapImplementationDSA(BufferAttachment attachment, GLenum textureTarget, GLuint textureId, GLint level);
         #endif
 
@@ -922,7 +919,6 @@ class MAGNUM_GL_EXPORT Framebuffer: public AbstractFramebuffer, public AbstractO
         #endif
         #ifndef MAGNUM_TARGET_GLES
         void MAGNUM_GL_LOCAL textureImplementationDSA(BufferAttachment attachment, GLuint textureId, GLint level);
-        void MAGNUM_GL_LOCAL textureImplementationDSAEXT(BufferAttachment attachment, GLuint textureId, GLint level);
         #endif
 
         #if !(defined(MAGNUM_TARGET_WEBGL) && defined(MAGNUM_TARGET_GLES2))
@@ -930,7 +926,6 @@ class MAGNUM_GL_EXPORT Framebuffer: public AbstractFramebuffer, public AbstractO
         #endif
         #ifndef MAGNUM_TARGET_GLES
         void MAGNUM_GL_LOCAL textureLayerImplementationDSA(BufferAttachment attachment, GLuint textureId, GLint level, GLint layer);
-        void MAGNUM_GL_LOCAL textureLayerImplementationDSAEXT(BufferAttachment attachment, GLuint textureId, GLint level, GLint layer);
         #endif
 };
 
@@ -943,17 +938,6 @@ inline GLuint Framebuffer::release() {
     return id;
 }
 
-}
-
-#ifdef MAGNUM_BUILD_DEPRECATED
-/* Note: needs to be prefixed with Magnum:: otherwise Doxygen can't find it */
-
-/** @brief @copybrief GL::Framebuffer
- * @deprecated Use @ref GL::Framebuffer instead.
- */
-typedef CORRADE_DEPRECATED("use GL::Framebuffer instead") Magnum::GL::Framebuffer Framebuffer;
-#endif
-
-}
+}}
 
 #endif
