@@ -3,7 +3,7 @@
 /*
     This file is part of Magnum.
 
-    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018
+    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019
               Vladimír Vondruš <mosra@centrum.cz>
 
     Permission is hereby granted, free of charge, to any person obtaining a
@@ -26,10 +26,10 @@
 */
 
 /** @file
- * @brief Class @ref Magnum::Math::DualQuaternion
+ * @brief Class @ref Magnum::Math::DualQuaternion, function @ref Magnum::Math::sclerp()
  */
 
-#include <cmath>
+#include <Corrade/Utility/StlMath.h>
 
 #include "Magnum/Math/Dual.h"
 #include "Magnum/Math/Functions.h"
@@ -48,33 +48,47 @@ namespace Implementation {
 @param normalizedB  Second dual quaternion
 @param t            Interpolation phase (from range @f$ [0; 1] @f$)
 
-Expects that both dual quaternions are normalized. @f[
-\begin{array}{rcl}
-    l + \epsilon m & = & \hat q_A^* \hat q_B \\
-    \frac{\hat a} 2 & = & acos \left( l_S \right) - \epsilon m_S \frac 1 {|l_V|} \\
-    \hat {\boldsymbol n} & = & \boldsymbol n_0 + \epsilon \boldsymbol n_\epsilon
-    ~~~~~~~~ \boldsymbol n_0 = l_V \frac 1 {|l_V|}
-    ~~~~~~~~ \boldsymbol n_\epsilon = \left( m_V - {\boldsymbol n}_0 \frac {a_\epsilon} 2 l_S \right)\frac 1 {|l_V|} \\
-    {\hat q}_{ScLERP} & = & \hat q_A (\hat q_A^* \hat q_B)^t =
-        \hat q_A \left[ \hat {\boldsymbol n} sin \left( t \frac {\hat a} 2 \right), cos \left( t \frac {\hat a} 2 \right) \right] \\
-\end{array}
+Expects that both dual quaternions are normalized. If the real parts are the
+same or one is a negation of the other, returns the @ref DualQuaternion::rotation()
+(real) part combined with interpolated @ref DualQuaternion::translation(): @f[
+    \begin{array}{rcl}
+        d & = & q_{A_0} \cdot q_{B_0} \\[5pt]
+        {\hat q}_{ScLERP} & = & 2 \left[(1 - t)(q_{A_\epsilon} q_{A_0}^*)_V + t (q_{B_\epsilon} q_{B_0}^*)_V \right] q_A, ~ {\color{m-primary} \text{if} ~ d \ge 1}
+    \end{array}
 @f]
-@see @ref DualQuaternion::isNormalized(),
-    @ref slerp(const Quaternion<T>&, const Quaternion<T>&, T),
-    @ref lerp(const T&, const T&, U)
+
+@m_class{m-noindent}
+
+otherwise, the interpolation is performed as: @f[
+    \begin{array}{rcl}
+        l + \epsilon m & = & \hat q_A^* \hat q_B \\[5pt]
+        \frac{\hat a} 2 & = & \arccos \left( l_S \right) - \epsilon m_S \frac 1 {|\boldsymbol{l}_V|} \\[5pt]
+        \hat {\boldsymbol n} & = & \boldsymbol n_0 + \epsilon \boldsymbol n_\epsilon,
+        ~~~~~~~~ \boldsymbol n_0 = \boldsymbol{l}_V \frac 1 {|\boldsymbol{l}_V|},
+        ~~~~~~~~ \boldsymbol n_\epsilon = \left(\boldsymbol{m}_V - {\boldsymbol n}_0 \frac {a_\epsilon} 2 l_S \right)\frac 1 {|\boldsymbol{l}_V|} \\[5pt]
+        {\hat q}_{ScLERP} & = & \hat q_A (\hat q_A^* \hat q_B)^t =
+            \hat q_A \left[ \hat {\boldsymbol n} \sin \left( t \frac {\hat a} 2 \right), \cos \left( t \frac {\hat a} 2 \right) \right]
+    \end{array}
+@f]
+
+Note that this function does not check for shortest path interpolation, see
+@ref sclerpShortestPath() for an alternative.
+@see @ref DualQuaternion::isNormalized(), @ref DualQuaternion::quaternionConjugated(),
+    @ref lerp(const Quaternion<T>&, const Quaternion<T>&, T),
+    @ref slerp(const Quaternion<T>&, const Quaternion<T>&, T)
 */
 template<class T> inline DualQuaternion<T> sclerp(const DualQuaternion<T>& normalizedA, const DualQuaternion<T>& normalizedB, const T t) {
     CORRADE_ASSERT(normalizedA.isNormalized() && normalizedB.isNormalized(),
-        "Math::sclerp(): dual quaternions must be normalized", {});
-    const T dotResult = dot(normalizedA.real().vector(), normalizedB.real().vector());
+        "Math::sclerp(): dual quaternions" << normalizedA << "and" << normalizedB << "are not normalized", {});
+    const T cosHalfAngle = dot(normalizedA.real(), normalizedB.real());
 
-    /* Avoid division by zero */
-    const T cosHalfAngle = dotResult + normalizedA.real().scalar()*normalizedB.real().scalar();
-    if(std::abs(cosHalfAngle) >= T(1))
-        return {normalizedA.real(), {Implementation::lerp(normalizedA.dual().vector(), normalizedB.dual().vector(), t), T(0)}};
+    /* Avoid division by zero: interpolate just the translation part */
+    /** @todo could this be optimized somehow? */
+    if(std::abs(cosHalfAngle) >= T(1) - TypeTraits<T>::epsilon())
+        return DualQuaternion<T>::translation(Implementation::lerp(normalizedA.translation(), normalizedB.translation(), t))*DualQuaternion<T>{normalizedA.real()};
 
-    /* l + εm = q_A^**q_B, multiplying with -1 ensures shortest path when dot < 0 */
-    const DualQuaternion<T> diff = normalizedA.quaternionConjugated()*(dotResult < T(0) ? -normalizedB : normalizedB);
+    /* l + εm = q_A^**q_B */
+    const DualQuaternion<T> diff = normalizedA.quaternionConjugated()*normalizedB;
     const Quaternion<T>& l = diff.real();
     const Quaternion<T>& m = diff.dual();
 
@@ -89,17 +103,88 @@ template<class T> inline DualQuaternion<T> sclerp(const DualQuaternion<T>& norma
     const Dual<Vector3<T>> n{direction, moment};
 
     /* q_ScLERP = q_A*(cos(t*a/2) + n*sin(t*a/2)) */
-    Dual<T> sin, cos;
-    std::tie(sin, cos) = Math::sincos(t*Dual<Rad<T>>(aHalf));
-    return normalizedA*DualQuaternion<T>{n*sin, cos};
+    const std::pair<Dual<T>, Dual<T>> sincos = Math::sincos(t*Dual<Rad<T>>(aHalf));
+    return normalizedA*DualQuaternion<T>{n*sincos.first, sincos.second};
+}
+
+/** @relatesalso DualQuaternion
+@brief Screw linear shortest-path interpolation of two dual quaternions
+@param normalizedA  First dual quaternion
+@param normalizedB  Second dual quaternion
+@param t            Interpolation phase (from range @f$ [0; 1] @f$)
+
+Unlike @ref sclerp(const DualQuaternion<T>&, const DualQuaternion<T>&, T) this
+function interpolates on the shortest path. Expects that both dual quaternions
+are normalized. If the real parts are the same or one is a negation of the
+other, returns the @ref DualQuaternion::rotation() (real) part combined with
+interpolated @ref DualQuaternion::translation(): @f[
+    \begin{array}{rcl}
+        d & = & q_{A_0} \cdot q_{B_0} \\[5pt]
+        {\hat q}_{ScLERP} & = & 2 \left((1 - t)(q_{A_\epsilon} q_{A_0}^*)_V + t (q_{B_\epsilon} q_{B_0}^*)_V \right) (q_{A_0} + \epsilon [\boldsymbol{0}, 0]), ~ {\color{m-primary} \text{if} ~ d \ge 1}
+    \end{array}
+@f]
+
+@m_class{m-noindent}
+
+otherwise, the interpolation is performed as: @f[
+    \begin{array}{rcl}
+        l + \epsilon m & = & \begin{cases}
+            \phantom{-}\hat q_A^* \hat q_B, & d \ge 0 \\
+            -\hat q_A^* \hat q_B, & d < 0 \\
+        \end{cases} \\[15pt]
+        \frac{\hat a} 2 & = & \arccos \left( l_S \right) - \epsilon m_S \frac 1 {|\boldsymbol{l}_V|} \\[5pt]
+        \hat {\boldsymbol n} & = & \boldsymbol n_0 + \epsilon \boldsymbol n_\epsilon,
+        ~~~~~~~~ \boldsymbol n_0 = \boldsymbol{l}_V \frac 1 {|\boldsymbol{l}_V|},
+        ~~~~~~~~ \boldsymbol n_\epsilon = \left(\boldsymbol{m}_V - {\boldsymbol n}_0 \frac {a_\epsilon} 2 l_S \right)\frac 1 {|\boldsymbol{l}_V|} \\[5pt]
+        {\hat q}_{ScLERP} & = & \hat q_A (\hat q_A^* \hat q_B)^t =
+            \hat q_A \left[ \hat {\boldsymbol n} \sin \left( t \frac {\hat a} 2 \right), \cos \left( t \frac {\hat a} 2 \right) \right]
+    \end{array}
+@f]
+@see @ref DualQuaternion::isNormalized(), @ref lerpShortestPath(),
+    @ref slerpShortestPath()
+*/
+template<class T> inline DualQuaternion<T> sclerpShortestPath(const DualQuaternion<T>& normalizedA, const DualQuaternion<T>& normalizedB, const T t) {
+    CORRADE_ASSERT(normalizedA.isNormalized() && normalizedB.isNormalized(),
+        "Math::sclerp(): dual quaternions" << normalizedA << "and" << normalizedB << "are not normalized", {});
+    const T cosHalfAngle = dot(normalizedA.real(), normalizedB.real());
+
+    /* Avoid division by zero: interpolate just the translation part */
+    /** @todo could this be optimized somehow? */
+    if(std::abs(cosHalfAngle) >= T(1) - TypeTraits<T>::epsilon())
+        return DualQuaternion<T>::translation(Implementation::lerp(normalizedA.translation(), normalizedB.translation(), t))*DualQuaternion<T>{normalizedA.real()};
+
+    /* l + εm = q_A^**q_B, multiplying with -1 ensures shortest path when dot < 0 */
+    const DualQuaternion<T> diff = normalizedA.quaternionConjugated()*(cosHalfAngle < T(0) ? -normalizedB : normalizedB);
+    const Quaternion<T>& l = diff.real();
+    const Quaternion<T>& m = diff.dual();
+
+    /* a/2 = acos(l_S) - εm_S/|l_V| */
+    const T invr = l.vector().lengthInverted();
+    const Dual<T> aHalf{std::acos(l.scalar()), -m.scalar()*invr};
+
+    /* direction = n_0 = l_V/|l_V|
+       moment = n_ε = (m_V - n_0*(a_ε/2)*l_S)/|l_V| */
+    const Vector3<T> direction = l.vector()*invr;
+    const Vector3<T> moment = (m.vector() - direction*(aHalf.dual()*l.scalar()))*invr;
+    const Dual<Vector3<T>> n{direction, moment};
+
+    /* q_ScLERP = q_A*(cos(t*a/2) + n*sin(t*a/2)) */
+    const std::pair<Dual<T>, Dual<T>> sincos = Math::sincos(t*Dual<Rad<T>>(aHalf));
+    return normalizedA*DualQuaternion<T>{n*sincos.first, sincos.second};
 }
 
 /**
 @brief Dual quaternion
 @tparam T   Underlying data type
 
-Represents 3D rotation and translation. See @ref transformations for brief
-introduction.
+Represents 3D rotation and translation. Usually denoted as the following in
+equations, with @f$ q_0 @f$ being the @ref real() part and @f$ q_\epsilon @f$
+the @ref dual() part: @f[
+    \hat q = q_0 + \epsilon q_\epsilon
+@f]
+
+See @ref Dual and @ref Quaternion for further notation description and
+@ref transformations for a brief introduction.
 @see @ref Magnum::DualQuaternion, @ref Magnum::DualQuaterniond, @ref Dual,
     @ref Quaternion, @ref Matrix4
 */
@@ -113,7 +198,7 @@ template<class T> class DualQuaternion: public Dual<Quaternion<T>> {
          * @param normalizedAxis    Normalized rotation axis
          *
          * Expects that the rotation axis is normalized. @f[
-         *      \hat q = [\boldsymbol a \cdot sin \frac \theta 2, cos \frac \theta 2] + \epsilon [\boldsymbol 0, 0]
+         *      \hat q = [\boldsymbol a \cdot \sin(\frac{\theta}{2}), \cos(\frac{\theta}{2})] + \epsilon [\boldsymbol 0, 0]
          * @f]
          * @see @ref rotation() const, @ref Quaternion::rotation(),
          *      @ref Matrix4::rotation(), @ref DualComplex::rotation(),
@@ -151,7 +236,7 @@ template<class T> class DualQuaternion: public Dual<Quaternion<T>> {
          */
         static DualQuaternion<T> fromMatrix(const Matrix4<T>& matrix) {
             CORRADE_ASSERT(matrix.isRigidTransformation(),
-                "Math::DualQuaternion::fromMatrix(): the matrix doesn't represent rigid transformation", {});
+                "Math::DualQuaternion::fromMatrix(): the matrix doesn't represent a rigid transformation:" << Corrade::Utility::Debug::newline << matrix, {});
 
             Quaternion<T> q = Implementation::quaternionFromMatrix(matrix.rotationScaling());
             return {q, Quaternion<T>(matrix.translation()/2)*q};
@@ -160,32 +245,24 @@ template<class T> class DualQuaternion: public Dual<Quaternion<T>> {
         /**
          * @brief Default constructor
          *
+         * Equivalent to @ref DualQuaternion(IdentityInitT).
+         */
+        constexpr /*implicit*/ DualQuaternion() noexcept: Dual<Quaternion<T>>{{}, {{}, T(0)}} {}
+
+        /**
+         * @brief Identity constructor
+         *
          * Creates unit dual quaternion. @f[
          *      \hat q = [\boldsymbol 0, 1] + \epsilon [\boldsymbol 0, 0]
          * @f]
          */
-        constexpr /*implicit*/ DualQuaternion(IdentityInitT = IdentityInit) noexcept
-            /** @todoc remove workaround when doxygen is sane */
-            #ifndef DOXYGEN_GENERATING_OUTPUT
-            : Dual<Quaternion<T>>{{}, {{}, T(0)}}
-            #endif
-            {}
+        constexpr explicit DualQuaternion(IdentityInitT) noexcept: Dual<Quaternion<T>>{{}, {{}, T(0)}} {}
 
         /** @brief Construct zero-initialized dual quaternion */
-        constexpr explicit DualQuaternion(ZeroInitT) noexcept
-            /** @todoc remove workaround when doxygen is sane */
-            #ifndef DOXYGEN_GENERATING_OUTPUT
-            : Dual<Quaternion<T>>{Quaternion<T>{ZeroInit}, Quaternion<T>{ZeroInit}}
-            #endif
-            {}
+        constexpr explicit DualQuaternion(ZeroInitT) noexcept: Dual<Quaternion<T>>{Quaternion<T>{ZeroInit}, Quaternion<T>{ZeroInit}} {}
 
         /** @brief Construct without initializing the contents */
-        explicit DualQuaternion(NoInitT) noexcept
-            /** @todoc remove workaround when doxygen is sane */
-            #ifndef DOXYGEN_GENERATING_OUTPUT
-            : Dual<Quaternion<T>>{NoInit}
-            #endif
-            {}
+        explicit DualQuaternion(NoInitT) noexcept: Dual<Quaternion<T>>{NoInit} {}
 
         /**
          * @brief Construct dual quaternion from real and dual part
@@ -203,11 +280,7 @@ template<class T> class DualQuaternion: public Dual<Quaternion<T>> {
          *      \hat q = [\hat{\boldsymbol v}, \hat s] = [\boldsymbol v_0, s_0] + \epsilon [\boldsymbol v_\epsilon, s_\epsilon]
          * @f]
          */
-        constexpr /*implicit*/ DualQuaternion(const Dual<Vector3<T>>& vector, const Dual<T>& scalar) noexcept
-            #ifndef DOXYGEN_GENERATING_OUTPUT
-            : Dual<Quaternion<T>>{{vector.real(), scalar.real()}, {vector.dual(), scalar.dual()}}
-            #endif
-            {}
+        constexpr /*implicit*/ DualQuaternion(const Dual<Vector3<T>>& vector, const Dual<T>& scalar) noexcept: Dual<Quaternion<T>>{{vector.real(), scalar.real()}, {vector.dual(), scalar.dual()}} {}
 
         /**
          * @brief Construct dual quaternion from vector
@@ -217,11 +290,7 @@ template<class T> class DualQuaternion: public Dual<Quaternion<T>> {
          * @f]
          * @see @ref transformPointNormalized()
          */
-        #ifdef DOXYGEN_GENERATING_OUTPUT
-        constexpr explicit DualQuaternion(const Vector3<T>& vector) noexcept;
-        #else
         constexpr explicit DualQuaternion(const Vector3<T>& vector) noexcept: Dual<Quaternion<T>>({}, {vector, T(0)}) {}
-        #endif
 
         /**
          * @brief Construct dual quaternion from another of different type
@@ -242,6 +311,15 @@ template<class T> class DualQuaternion: public Dual<Quaternion<T>> {
             return Implementation::DualQuaternionConverter<T, U>::to(*this);
         }
 
+        /**
+         * @brief Raw data
+         * @return One-dimensional array of eight elements
+         *
+         * @see @ref real(), @ref dual()
+         */
+        T* data() { return Dual<Quaternion<T>>::data()->data(); }
+        constexpr const T* data() const { return Dual<Quaternion<T>>::data()->data(); } /**< @overload */
+
         /**
          * @brief Whether the dual quaternion is normalized
          *
@@ -385,7 +463,7 @@ template<class T> class DualQuaternion: public Dual<Quaternion<T>> {
          */
         DualQuaternion<T> invertedNormalized() const {
             CORRADE_ASSERT(isNormalized(),
-                           "Math::DualQuaternion::invertedNormalized(): dual quaternion must be normalized", {});
+                "Math::DualQuaternion::invertedNormalized():" << *this << "is not normalized", {});
             return quaternionConjugated();
         }
 
@@ -419,7 +497,7 @@ template<class T> class DualQuaternion: public Dual<Quaternion<T>> {
          */
         Vector3<T> transformPointNormalized(const Vector3<T>& vector) const {
             CORRADE_ASSERT(isNormalized(),
-                           "Math::DualQuaternion::transformPointNormalized(): dual quaternion must be normalized", {});
+                "Math::DualQuaternion::transformPointNormalized():" << *this << "is not normalized", {});
             return ((*this)*DualQuaternion<T>(vector)*conjugated()).dual().vector();
         }
 
@@ -429,7 +507,8 @@ template<class T> class DualQuaternion: public Dual<Quaternion<T>> {
 
 MAGNUM_DUAL_OPERATOR_IMPLEMENTATION(DualQuaternion, Quaternion, T)
 
-/** @debugoperator{Magnum::Math::DualQuaternion} */
+#ifndef CORRADE_NO_DEBUG
+/** @debugoperator{DualQuaternion} */
 template<class T> Corrade::Utility::Debug& operator<<(Corrade::Utility::Debug& debug, const DualQuaternion<T>& value) {
     return debug << "DualQuaternion({{" << Corrade::Utility::Debug::nospace
         << value.real().vector().x() << Corrade::Utility::Debug::nospace << ","
@@ -448,6 +527,11 @@ template<class T> Corrade::Utility::Debug& operator<<(Corrade::Utility::Debug& d
 extern template MAGNUM_EXPORT Corrade::Utility::Debug& operator<<(Corrade::Utility::Debug&, const DualQuaternion<Float>&);
 extern template MAGNUM_EXPORT Corrade::Utility::Debug& operator<<(Corrade::Utility::Debug&, const DualQuaternion<Double>&);
 #endif
+#endif
+
+namespace Implementation {
+    template<class T> struct StrictWeakOrdering<DualQuaternion<T>>: StrictWeakOrdering<Dual<Quaternion<T>>> {};
+}
 
 }}
 
