@@ -3,7 +3,7 @@
 /*
     This file is part of Magnum.
 
-    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018
+    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019
               Vladimír Vondruš <mosra@centrum.cz>
 
     Permission is hereby granted, free of charge, to any person obtaining a
@@ -32,10 +32,6 @@
 #include "Magnum/Primitives/visibility.h"
 #include "Magnum/Trade/Trade.h"
 
-#ifdef MAGNUM_BUILD_DEPRECATED
-#include <Corrade/Utility/Macros.h>
-#endif
-
 namespace Magnum { namespace Primitives {
 
 /**
@@ -56,19 +52,6 @@ Indexed @ref MeshPrimitive::Lines.
 */
 MAGNUM_PRIMITIVES_EXPORT Trade::MeshData2D capsule2DWireframe(UnsignedInt hemisphereRings, UnsignedInt cylinderRings, Float halfLength);
 
-#ifdef MAGNUM_BUILD_DEPRECATED
-/**
-@brief 2D capsule
-@deprecated Use @ref capsule2DWireframe() instead.
-*/
-struct MAGNUM_PRIMITIVES_EXPORT Capsule2D {
-    /** @copybrief capsule2DWireframe()
-     * @deprecated Use @ref capsule2DWireframe() instead.
-     */
-    CORRADE_DEPRECATED("use capsule2DWireframe() instead") static Trade::MeshData2D wireframe(UnsignedInt hemisphereRings, UnsignedInt cylinderRings, Float halfLength);
-};
-#endif
-
 /**
 @brief Whether to generate capsule texture coordinates
 
@@ -125,29 +108,6 @@ Indexed @ref MeshPrimitive::Lines.
 */
 MAGNUM_PRIMITIVES_EXPORT Trade::MeshData3D capsule3DWireframe(UnsignedInt hemisphereRings, UnsignedInt cylinderRings, UnsignedInt segments, Float halfLength);
 
-#ifdef MAGNUM_BUILD_DEPRECATED
-/**
-@brief 3D capsule
-@deprecated Use @ref capsule3DSolid() or @ref capsule3DWireframe() instead.
-*/
-struct MAGNUM_PRIMITIVES_EXPORT Capsule3D {
-    /** @brief @copybrief CapsuleTextureCoords
-     * @deprecated Use @ref CapsuleTextureCoords instead.
-     */
-    typedef CORRADE_DEPRECATED("use CapsuleTextureCoords instead") CapsuleTextureCoords TextureCoords;
-
-    /** @brief @copybrief capsule3DSolid()
-     * @deprecated Use @ref capsule3DSolid() instead.
-     */
-    CORRADE_DEPRECATED("use capsule3DSolid() instead") static Trade::MeshData3D solid(UnsignedInt hemisphereRings, UnsignedInt cylinderRings, UnsignedInt segments, Float halfLength, CapsuleTextureCoords textureCoords = CapsuleTextureCoords::DontGenerate);
-
-    /** @brief @copybrief capsule3DWireframe()
-     * @deprecated Use @ref capsule3DWireframe() instead.
-     */
-    CORRADE_DEPRECATED("use capsule3DWireframe() instead") static Trade::MeshData3D wireframe(UnsignedInt hemisphereRings, UnsignedInt cylinderRings, UnsignedInt segments, Float halfLength);
-};
-#endif
-
 }}
 
 #endif
