@@ -27,7 +27,6 @@
 
 #define MAGNUM_BUILD_DEPRECATED
 /* #undef MAGNUM_BUILD_STATIC */
-#define MAGNUM_BUILD_MULTITHREADED
 #define MAGNUM_TARGET_GL
 /* #undef MAGNUM_TARGET_GLES */
 /* #undef MAGNUM_TARGET_GLES2 */
@@ -37,4 +36,12 @@
 /* #undef MAGNUM_TARGET_HEADLESS */
 #define MAGNUM_TARGET_VK
 
+#ifdef MAGNUM_BUILD_DEPRECATED
+#include "Corrade/configure.h"
+#ifdef CORRADE_BUILD_MULTITHREADED
+/* For compatibility only, to be removed at some point */
+#define MAGNUM_BUILD_MULTITHREADED
+#endif
+#endif
+
 #endif
